/*
*  Rubrica
*  file: dictionary.c
*
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "dictionary.h"
#include "prefer.h"



Dictionary dict;
gboolean have_dict = FALSE;

RubricaDictionary rubrica_dict[] = {
  {ADDRESS, "Indirizzo", "Address", "" },
  {CARD, "Scheda", "Card", "" },
  {MODIFYING, "editabile", "modifying", "" },
  {DELETING, "cancellabile", "deleting", "" },
  {PERSONAL_DATA, "Dati", "Data", "" },
  {FIRST_NAME, "Nome", "FirstName", "" },
  {LAST_NAME, "Cognome", "LastName", "" },
  {PROFESSION, "Professione", "Profession", "" },
  {STREET, "Via", "Street", "" },
  {STREET_NUMBER, "Numero", "StreetNumber", "" },
  {ZIP_CODE, "Cap", "ZipCode", "" },
  {CITY, "Citta", "City", "" },
  {PROVINCE, "Provincia", "Province", ""},
  {COUNTRY, "Stato", "Country", ""},
  {WEB_ADDRESS, "IndirizziWeb", "WebAddresses", ""},
  {WEB, "Web", "Web", ""},
  {EMAIL_ADDRESS, "IndirizziEmail", "EmailAddresses", ""},
  {EMAIL, "EMail", "EMail", ""},
  {TELEPHONE_NUMBERS, "NumeriTelefonici", "TelephoneNumbers", ""},
  {TELEPHONE, "Telefono", "Telephone", ""},
  {TYPE, "Tipo", "Type", ""},
  {PUBLIC_KEY, "ChiavePublica", "PublicKey", ""},
  {NOTE, "Note", "Notes", ""},
  {END, NULL, NULL, NULL}
};

void rubrica_init_dictionary()
{
  LANG lang;
  
  if (!preferences_loaded())
    {
      g_warning("preferences not loaded. I will use default english dictionary");
      lang = ENGLISH;
    }
  
  lang = preferences_get_lang_code();

  switch (lang)
    {
    case ITALIAN:
      dict_load_it();
      have_dict = TRUE;
      break;

    case ENGLISH:
      dict_load_en();
      have_dict = TRUE;
      break; 

    case OTHER:
      dict_load_oth();
      have_dict = TRUE;
      break;  

    default:
      break;
    }
}

void dict_load_it(void)
{
  if (have_dict)
    dict_clean_dictionary();

  dict.address  = g_strdup(rubrica_dict[0].itstr);
  dict.card     = g_strdup(rubrica_dict[1].itstr);
  dict.mod      = g_strdup(rubrica_dict[2].itstr);
  dict.del      = g_strdup(rubrica_dict[3].itstr);
  dict.data     = g_strdup(rubrica_dict[4].itstr);
  dict.name     = g_strdup(rubrica_dict[5].itstr);
  dict.surn     = g_strdup(rubrica_dict[6].itstr);
  dict.prof     = g_strdup(rubrica_dict[7].itstr);
  dict.str      = g_strdup(rubrica_dict[8].itstr);
  dict.num      = g_strdup(rubrica_dict[9].itstr);
  dict.zip      = g_strdup(rubrica_dict[10].itstr);
  dict.city     = g_strdup(rubrica_dict[11].itstr);
  dict.prov     = g_strdup(rubrica_dict[12].itstr);
  dict.st       = g_strdup(rubrica_dict[13].itstr);
  dict.webadd   = g_strdup(rubrica_dict[14].itstr);
  dict.web      = g_strdup(rubrica_dict[15].itstr);
  dict.mailadd  = g_strdup(rubrica_dict[16].itstr);
  dict.mail     = g_strdup(rubrica_dict[17].itstr);
  dict.tellist  = g_strdup(rubrica_dict[18].itstr);
  dict.telnum   = g_strdup(rubrica_dict[19].itstr);
  dict.teltype  = g_strdup(rubrica_dict[20].itstr);
  dict.pubkey   = g_strdup(rubrica_dict[21].itstr);
  dict.note     = g_strdup(rubrica_dict[22].itstr);
}

void dict_load_en(void)
{
  if(have_dict)
    dict_clean_dictionary();

  dict.address  = g_strdup(rubrica_dict[0].enstr);
  dict.card     = g_strdup(rubrica_dict[1].enstr);
  dict.mod      = g_strdup(rubrica_dict[2].enstr);
  dict.del      = g_strdup(rubrica_dict[3].enstr);
  dict.data     = g_strdup(rubrica_dict[4].enstr);
  dict.name     = g_strdup(rubrica_dict[5].enstr);
  dict.surn     = g_strdup(rubrica_dict[6].enstr);
  dict.prof     = g_strdup(rubrica_dict[7].enstr);
  dict.str      = g_strdup(rubrica_dict[8].enstr);
  dict.num      = g_strdup(rubrica_dict[9].enstr);
  dict.zip      = g_strdup(rubrica_dict[10].enstr);
  dict.city     = g_strdup(rubrica_dict[11].enstr);
  dict.prov     = g_strdup(rubrica_dict[12].enstr);
  dict.st       = g_strdup(rubrica_dict[13].enstr);
  dict.webadd   = g_strdup(rubrica_dict[14].enstr);
  dict.web      = g_strdup(rubrica_dict[15].enstr);
  dict.mailadd  = g_strdup(rubrica_dict[16].enstr);
  dict.mail     = g_strdup(rubrica_dict[17].enstr);
  dict.tellist  = g_strdup(rubrica_dict[18].enstr);
  dict.telnum   = g_strdup(rubrica_dict[19].enstr);
  dict.teltype  = g_strdup(rubrica_dict[20].enstr);
  dict.pubkey   = g_strdup(rubrica_dict[21].enstr);
  dict.note     = g_strdup(rubrica_dict[22].enstr);
}

void dict_load_oth(void)
{
  if (have_dict)
    dict_clean_dictionary();

  dict.address  = g_strdup(rubrica_dict[0].trans);
  dict.card     = g_strdup(rubrica_dict[1].trans);
  dict.mod      = g_strdup(rubrica_dict[2].trans);
  dict.del      = g_strdup(rubrica_dict[3].trans);
  dict.data     = g_strdup(rubrica_dict[4].trans);
  dict.name     = g_strdup(rubrica_dict[5].trans);
  dict.surn     = g_strdup(rubrica_dict[6].trans);
  dict.prof     = g_strdup(rubrica_dict[7].trans);
  dict.str      = g_strdup(rubrica_dict[8].trans);
  dict.num      = g_strdup(rubrica_dict[9].trans);
  dict.zip      = g_strdup(rubrica_dict[10].trans);
  dict.city     = g_strdup(rubrica_dict[11].trans);
  dict.prov     = g_strdup(rubrica_dict[12].trans);
  dict.st       = g_strdup(rubrica_dict[13].trans);
  dict.webadd   = g_strdup(rubrica_dict[14].trans);
  dict.web      = g_strdup(rubrica_dict[15].trans);
  dict.mailadd  = g_strdup(rubrica_dict[16].trans);
  dict.mail     = g_strdup(rubrica_dict[17].trans);
  dict.tellist  = g_strdup(rubrica_dict[18].trans);
  dict.telnum   = g_strdup(rubrica_dict[19].trans);
  dict.teltype  = g_strdup(rubrica_dict[20].trans);
  dict.pubkey   = g_strdup(rubrica_dict[21].trans);
  dict.note     = g_strdup(rubrica_dict[22].trans);
}

void dict_clean_dictionary()
{
  g_free(dict.address);  
  g_free(dict.card);
  g_free(dict.mod);
  g_free(dict.del);
  g_free(dict.data);     
  g_free(dict.name);    
  g_free(dict.surn);     
  g_free(dict.prof);     
  g_free(dict.str);      
  g_free(dict.num);     
  g_free(dict.zip);     
  g_free(dict.city);     
  g_free(dict.prov);     
  g_free(dict.st);       
  g_free(dict.webadd);   
  g_free(dict.web);      
  g_free(dict.mailadd);  
  g_free(dict.mail);     
  g_free(dict.telnum);   
  g_free(dict.teltype);  
  g_free(dict.pubkey);   
  g_free(dict.note);

  have_dict = FALSE;
}


void rubrica_load_translation()
{
  LANG language;

  if (!preferences_get_use_dictionary)
    {      
      language = preferences_get_lang_code();

      if (language == ITALIAN)
	dict_load_it();
      else
	dict_load_en();
    }
  dict_load_oth();
}

Dictionary rubrica_get_dictionary(gchar *sh_lang)
{
  if (sh_lang == NULL)
      return (Dictionary) dict;

  if (g_strcasecmp(sh_lang, "IT") == 0)
      dict_load_it();      
  else
    if (g_strcasecmp(sh_lang, "EN") == 0)
      dict_load_en();     

  return (Dictionary) dict; 
}

void rubrica_store_dictionary(Dictionary dict)
{
  rubrica_dict[0].trans = g_strdup(dict.address);
  rubrica_dict[1].trans = g_strdup(dict.card);
  rubrica_dict[2].trans = g_strdup(dict.mod);
  rubrica_dict[3].trans = g_strdup(dict.del);
  rubrica_dict[4].trans = g_strdup(dict.data);
  rubrica_dict[5].trans = g_strdup(dict.name);
  rubrica_dict[6].trans = g_strdup(dict.surn);
  rubrica_dict[7].trans = g_strdup(dict.prof);
  rubrica_dict[8].trans = g_strdup(dict.str);
  rubrica_dict[9].trans = g_strdup(dict.num);
  rubrica_dict[10].trans = g_strdup(dict.zip);
  rubrica_dict[11].trans = g_strdup(dict.city);
  rubrica_dict[12].trans = g_strdup(dict.prov);
  rubrica_dict[13].trans = g_strdup(dict.st);
  rubrica_dict[14].trans = g_strdup(dict.webadd);
  rubrica_dict[15].trans = g_strdup(dict.web);
  rubrica_dict[16].trans = g_strdup(dict.mailadd);
  rubrica_dict[17].trans = g_strdup(dict.mail);
  rubrica_dict[18].trans = g_strdup(dict.tellist);
  rubrica_dict[19].trans = g_strdup(dict.telnum);
  rubrica_dict[20].trans = g_strdup(dict.teltype);
  rubrica_dict[21].trans = g_strdup(dict.pubkey);
  rubrica_dict[22].trans = g_strdup(dict.note);
}


/*
  return TRUE if an error has occurred
  return FALSE if error hasn't occurred
 */
gboolean rubrica_load_user_dictionary(gchar *lang, gchar **shlang)
{
  xmlDocPtr doc;
  xmlNodePtr node;
  gchar *filename;
  gchar *home, *buffer;
  gchar *dict_dir;
  
  if ( (g_strcasecmp(lang, "italian") == 0) || (g_strcasecmp(lang, "english") == 0))   
    return FALSE;
  
  home = g_get_home_dir();
  dict_dir = g_strdup_printf("%s/.rubrica/dictionary", home);
  
  if (!g_file_exists(dict_dir))
    mkdir(dict_dir, S_IRUSR | S_IWUSR | S_IXUSR);
      
  filename = g_strdup_printf("%s/.rubrica/dictionary/%s.dict",home, lang);
 
  if (!g_file_exists(filename))
    {
      g_warning(_("The requested dictionary, %s.dict, don't exist."), lang);
      return TRUE;
    }
  
  doc = xmlParseFile(filename);
  if (!doc)
    {
      g_warning(_("I can't read the %s.dict dictionary"),lang);      
      return TRUE;
    }

   if (!doc->root || !doc->root->name || 
      g_strcasecmp(doc->root->name, "Rubrica_dictionary") != 0)
    {
      g_warning(_("This dictionary is corrupted\nHeader don't match with \"Rubrica_dictionary\"\nI can't read this dictionary"));
      xmlFreeDoc(doc);
      return TRUE;
    } 

   *shlang = xmlNodeGetLang(doc->root); 
   
   node = doc->root->childs;
   
   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.address = g_strdup(buffer);
   else 
     dict.address = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.card = g_strdup(buffer);
   else 
     dict.card = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.mod = g_strdup(buffer);
   else 
     dict.mod = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.del = g_strdup(buffer);
   else 
     dict.del = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.data = g_strdup(buffer);
   else 
     dict.data = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.name = g_strdup(buffer);
   else 
     dict.name = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.surn = g_strdup(buffer);
   else 
     dict.surn = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.prof = g_strdup(buffer);
   else 
     dict.prof = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.str = g_strdup(buffer);
   else 
     dict.str = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.num = g_strdup(buffer);
   else 
     dict.num = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.zip = g_strdup(buffer);
   else 
     dict.zip = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.city = g_strdup(buffer);
   else 
     dict.city = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.prov = g_strdup(buffer);
   else 
     dict.prov = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.st = g_strdup(buffer);
   else 
     dict.st = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.webadd = g_strdup(buffer);
   else 
     dict.webadd = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.web = g_strdup(buffer);
   else 
     dict.web = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.mailadd = g_strdup(buffer);
   else 
     dict.mailadd = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.mail = g_strdup(buffer);
   else 
     dict.mail = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.tellist = g_strdup(buffer);
   else 
     dict.tellist = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.telnum = g_strdup(buffer);
   else 
     dict.telnum = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.teltype = g_strdup(buffer);
   else 
     dict.teltype = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.pubkey = g_strdup(buffer);
   else 
     dict.pubkey = "";
   node = node->next;

   buffer = xmlNodeGetContent(node);
   if(buffer)
     dict.note = g_strdup(buffer);
   else 
     dict.note = "";
   node = node->next;

   xmlFreeDoc(doc);
   
   return FALSE;
}

gboolean rubrica_save_user_dictionary(Dictionary userdict, LANG l, 
				      gchar *lang, gchar *shlang)
{
  xmlDocPtr doc;
  xmlNodePtr node;
  gchar *filename;
  gchar *home, *dict_dir;

  home = g_get_home_dir();
  dict_dir = g_strdup_printf("%s/.rubrica/dictionary", home);

  if (!g_file_exists(dict_dir))
      mkdir(dict_dir, S_IRUSR | S_IWUSR | S_IXUSR);
  g_free(dict_dir);

  filename = g_strdup_printf("%s/.rubrica/dictionary/%s.dict",home, lang);
  doc = xmlNewDoc("1.0");

  doc->root = xmlNewDocNode(doc, NULL, "Rubrica_dictionary", NULL);
  xmlNodeSetLang(doc->root,shlang);

  node = xmlNewChild(doc->root, NULL, "Address",userdict.address);
  node = xmlNewChild(doc->root, NULL, "Card", userdict.card);
  node = xmlNewChild(doc->root, NULL, "Modifying", userdict.mod);
  node = xmlNewChild(doc->root, NULL, "Deleting", userdict.del);
  node = xmlNewChild(doc->root, NULL, "Date", userdict.data);
  node = xmlNewChild(doc->root, NULL, "FirstName", userdict.name);
  node = xmlNewChild(doc->root, NULL, "LastName", userdict.surn);
  node = xmlNewChild(doc->root, NULL, "Profession", userdict.prof);
  node = xmlNewChild(doc->root, NULL, "Street", userdict.str);
  node = xmlNewChild(doc->root, NULL, "StreetNumber", userdict.num);
  node = xmlNewChild(doc->root, NULL, "ZipCode", userdict.zip);
  node = xmlNewChild(doc->root, NULL, "City", userdict.city);
  node = xmlNewChild(doc->root, NULL, "Province", userdict.prov);
  node = xmlNewChild(doc->root, NULL, "Country", userdict.st);
  node = xmlNewChild(doc->root, NULL, "WebAddress", userdict.webadd);
  node = xmlNewChild(doc->root, NULL, "Web", userdict.web);
  node = xmlNewChild(doc->root, NULL, "EMailAddress", userdict.mailadd);
  node = xmlNewChild(doc->root, NULL, "Email", userdict.mail);
  node = xmlNewChild(doc->root, NULL, "TelephoneNumbers", userdict.tellist);
  node = xmlNewChild(doc->root, NULL, "Telephone", userdict.telnum);
  node = xmlNewChild(doc->root, NULL, "TelephoneType", userdict.teltype);
  node = xmlNewChild(doc->root, NULL, "PublicKey", userdict.pubkey);
  node = xmlNewChild(doc->root, NULL, "Notes", userdict.note);

  if( xmlSaveFile(filename, doc) == -1)
    {
      g_warning("can't save dictionary");
      
      return FALSE;
    }

  xmlFreeDoc(doc);
  g_free(filename);
  g_free(home);

  return TRUE;
}

/*  copy src into dest
 */
void dict_write(gchar *src, gchar **dest)
{
  if (src)
    *dest = g_strdup(src);
  else
    dest = NULL;     
}


void rubrica_extract_user_dictionary(Dictionary *dict, gpointer pbox)
{
  GtkWidget *eabook, *ecard, *edel, *emod, *edata, *efnname, *ename,
    *ecountry, *eprov, *ecity, *ezip, *estret, *estreetn, *eprof, 
    *ewebadd, *eweb, *emailadd, *email, 
    *etelnum, *tellst, *eteltype,
    *epubk, *enote;
  gchar *buffer = NULL;
  
  eabook   = gtk_object_get_data(GTK_OBJECT(pbox), "entry_address");
  ecard    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_card");
  edel     = gtk_object_get_data(GTK_OBJECT(pbox), "entry_del");
  emod     = gtk_object_get_data(GTK_OBJECT(pbox), "entry_mod");
  edata    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_data");
  efnname  = gtk_object_get_data(GTK_OBJECT(pbox), "entry_fn");
  ename    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_n");
  ecountry = gtk_object_get_data(GTK_OBJECT(pbox), "entry_country");
  eprov    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_prov");
  ecity    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_city");
  ezip     = gtk_object_get_data(GTK_OBJECT(pbox), "entry_zip");
  estret   = gtk_object_get_data(GTK_OBJECT(pbox), "entry_stn");
  estreetn = gtk_object_get_data(GTK_OBJECT(pbox), "entry_st");
  eprof    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_prof");
  ewebadd  = gtk_object_get_data(GTK_OBJECT(pbox), "entry_web_add");
  eweb     = gtk_object_get_data(GTK_OBJECT(pbox), "entry_web");
  emailadd = gtk_object_get_data(GTK_OBJECT(pbox), "entry_mail_add");
  email    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_mail");
  etelnum  = gtk_object_get_data(GTK_OBJECT(pbox), "entry_tel_num");
  tellst   = gtk_object_get_data(GTK_OBJECT(pbox), "entry_tel_lst");
  eteltype = gtk_object_get_data(GTK_OBJECT(pbox), "entry_tel_type");
  epubk    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_pubk");
  enote    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_notes");
  
  buffer = gtk_entry_get_text(GTK_ENTRY(eabook));
  dict_write(buffer, &dict->address);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(ecard));
  dict_write(buffer, &dict->card);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(emod));
  dict_write(buffer,  &dict->mod);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(edel));
  dict_write(buffer,  &dict->del);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(edata));
  dict_write(buffer,  &dict->data);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(ename));
  dict_write(buffer,  &dict->name);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(efnname));
  dict_write(buffer,  &dict->surn);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(eprof));
  dict_write(buffer,  &dict->prof);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(estret));
  dict_write(buffer,  &dict->str);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(estreetn));
  dict_write(buffer,  &dict->num);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(ezip));
  dict_write(buffer, &dict->zip);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(ecity));
  dict_write(buffer, &dict->city);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(eprov));
  dict_write(buffer,  &dict->prov);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(ecountry));
  dict_write(buffer,  &dict->st);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(ewebadd));
  dict_write(buffer,  &dict->webadd);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(eweb));
  dict_write(buffer,  &dict->web);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(emailadd));
  dict_write(buffer,  &dict->mailadd);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(email));
  dict_write(buffer,  &dict->mail);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(tellst));
  dict_write(buffer,  &dict->tellist);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(etelnum));
  dict_write(buffer,  &dict->telnum);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(eteltype));
  dict_write(buffer,  &dict->teltype);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(epubk));
  dict_write(buffer,  &dict->pubkey);
  buffer = NULL;

  buffer = gtk_entry_get_text(GTK_ENTRY(enote));
  dict_write(buffer,  &dict->note);
  buffer = NULL;

  rubrica_store_dictionary(*dict);
}

void rubrica_display_user_dictionary(gpointer pbox)
{
  GtkWidget *eabook, *ecard, *edel, *emod, *edata, *efnname, *ename,
    *ecountry, *eprov, *ecity, *ezip, *estret, *estreetn, *eprof, 
    *ewebadd, *eweb, *emailadd, *email, 
    *etelnum, *tellst, *eteltype,
    *epubk, *enote;
  
  eabook   = gtk_object_get_data(GTK_OBJECT(pbox), "entry_address");
  ecard    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_card");
  edel     = gtk_object_get_data(GTK_OBJECT(pbox), "entry_del");
  emod     = gtk_object_get_data(GTK_OBJECT(pbox), "entry_mod");
  edata    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_data");
  efnname  = gtk_object_get_data(GTK_OBJECT(pbox), "entry_fn");
  ename    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_n");
  ecountry = gtk_object_get_data(GTK_OBJECT(pbox), "entry_country");
  eprov    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_prov");
  ecity    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_city");
  ezip     = gtk_object_get_data(GTK_OBJECT(pbox), "entry_zip");
  estret   = gtk_object_get_data(GTK_OBJECT(pbox), "entry_stn");
  estreetn = gtk_object_get_data(GTK_OBJECT(pbox), "entry_st");
  eprof    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_prof");
  ewebadd  = gtk_object_get_data(GTK_OBJECT(pbox), "entry_web_add");
  eweb     = gtk_object_get_data(GTK_OBJECT(pbox), "entry_web");
  emailadd = gtk_object_get_data(GTK_OBJECT(pbox), "entry_mail_add");
  email    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_mail");
  etelnum  = gtk_object_get_data(GTK_OBJECT(pbox), "entry_tel_num");
  tellst   = gtk_object_get_data(GTK_OBJECT(pbox), "entry_tel_lst");
  eteltype = gtk_object_get_data(GTK_OBJECT(pbox), "entry_tel_type");
  epubk    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_pubk");
  enote    = gtk_object_get_data(GTK_OBJECT(pbox), "entry_notes");  

  gtk_entry_set_text(GTK_ENTRY(eabook), dict.address );
  gtk_entry_set_text(GTK_ENTRY(ecard), dict.card );
  gtk_entry_set_text(GTK_ENTRY(edel), dict.del );
  gtk_entry_set_text(GTK_ENTRY(emod), dict.mod );
  gtk_entry_set_text(GTK_ENTRY(edata), dict.data );
  gtk_entry_set_text(GTK_ENTRY(efnname), dict.name );
  gtk_entry_set_text(GTK_ENTRY(ename), dict.surn );
  gtk_entry_set_text(GTK_ENTRY(ecountry), dict.st );
  gtk_entry_set_text(GTK_ENTRY(eprov), dict.prov );
  gtk_entry_set_text(GTK_ENTRY(ecity), dict.city );
  gtk_entry_set_text(GTK_ENTRY(ezip), dict.zip );
  gtk_entry_set_text(GTK_ENTRY(estret), dict.str );
  gtk_entry_set_text(GTK_ENTRY(estreetn), dict.num );
  gtk_entry_set_text(GTK_ENTRY(eprof), dict.prof );
  gtk_entry_set_text(GTK_ENTRY(ewebadd), dict.webadd );
  gtk_entry_set_text(GTK_ENTRY(eweb), dict.web );
  gtk_entry_set_text(GTK_ENTRY(emailadd), dict.mailadd );
  gtk_entry_set_text(GTK_ENTRY(email), dict.mail );
  gtk_entry_set_text(GTK_ENTRY(etelnum), dict.telnum );
  gtk_entry_set_text(GTK_ENTRY(tellst), dict.tellist );
  gtk_entry_set_text(GTK_ENTRY(eteltype), dict.teltype );
  gtk_entry_set_text(GTK_ENTRY(epubk), dict.pubkey );
  gtk_entry_set_text(GTK_ENTRY(enote), dict.note );
}

