/*
*  Rubrica
*  file: file.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "rubrica.h"
#include "modulo.h"
#include "main.h"
#include "file.h"
#include "prefer.h"
#include "dictionary.h"
#include "addressbook.h"

VistaDati vista;
gchar *path;
gchar *nomefile;
Dictionary dict;
gchar *language;
gboolean language_tested = FALSE;
gboolean recover = FALSE;
gboolean start = FALSE;

gboolean rubrica_file_is_start(void)
{
  return start;
}

void rubrica_file_set_start(gboolean state)
{
  start = state;
}



void rubrica_file_set_names(gchar *pathfile, gchar *filename)
{
  if (path)
    g_free(path);
  if (nomefile)
    g_free(nomefile);

  path = g_strdup(pathfile);
  nomefile = g_strdup(filename);   
}


void rubrica_file_init_names(void)
{
  if (path)
    g_free(path);
  if (nomefile)
    g_free(nomefile);

  path = NULL;
  nomefile = NULL;
}

void rubrica_do_backup(gchar *old, gchar *new)
{
  /* se esiste gi un file con lo stesso nome, crea una copia di backup 
     if exist a file with same name, then do a backup of old file */
  if (g_file_exists(new))
    {
      gchar *ext=NULL, *f=NULL;
      gchar *bfile;
      gint num;

      rubrica_file_split(new, f, ext, &num);

      bfile = g_strdup_printf("%s.%s%d",f,ext,num);
      rubrica_do_backup(old, bfile);
      g_free(bfile);
    }
  else
    rename(old, new);  
}

void rubrica_file_split(gchar *file, gchar *f, gchar *ext, gint *n)
{
  gchar *buf, *tmp;
  gint i = 0, num = 0;

  f = g_basename(file);
  tmp = g_strdup(f);
  buf = tmp;
  while(*buf != '.')
    {
      buf++;
      i++;
    }

  if (buf)
    strcpy(ext, buf);

  g_free(tmp);
   
  tmp = ext;
  g_strreverse(tmp);
  i = 0;
  num = 0;
  while (tmp)
    {
      if (isdigit(*tmp))
	{
	  num = num * pow(10,i);
	  i++;
	}	
      tmp++;
    }
  num+=1;
  
  *n = num;
}

/* ***************************** */




void 
rubrica_salva_file(GtkWidget *w, gpointer data)
{
  GtkWidget *app;
  gchar *tmp;
  gboolean user_dict;

  app = rubrica_get_app();

  /* if user has his dictionary */
  user_dict = preferences_get_use_dictionary();
  if (user_dict)
    {
      if(!dictionary_is_loaded())
	rubrica_load_translation();  
      
      /* get dictionary */
      dict = rubrica_get_dictionary(NULL);
    }
  
  /* 
     se non c' un nome file di default allora bisogna richiamare salva_con_nome     
     if rubrica_salva_file is called without a file name, then call save as
  */
  if (!nomefile || g_strcasecmp(nomefile, _("no name")) == 0)
    /* salvo come default? */
    rubrica_salva_file_come(NULL, NULL);

    /*   altrimenti provo a scrivere il file 
    */
  else 
    if (!write_xml_file(nomefile))
      {
	tmp = g_strdup_printf(_("I can't save the file %s"),nomefile);
	gnome_app_error(GNOME_APP(app), tmp);
	g_free(tmp);
      }
}

void 
rubrica_salva_file_come(GtkWidget *w, gpointer data)
{
  GtkWidget *app;
  GtkFileSelection *fs;
  RubricaPreferences *p;
  
  app = rubrica_get_app();

  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Save as")));
  
  /* se ho gi un nome di file lo mostro nell'entry */
  if (nomefile)
       gtk_file_selection_set_filename(fs, nomefile);

  /* altrimenti, se settato, propongo il nome di default */
  else
    {
      p = rubrica_get_preferences();
      
      if (p->ab_name)
	gtk_file_selection_set_filename(fs, p->ab_name);
    }

  /* connetto i bottoni ok e cancella */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_scrivi_file), fs);
  
  /* con il bottone cancella distruggo direttamente il widget */
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));
  
  gtk_widget_show(GTK_WIDGET(fs));
}


void
rubrica_scrivi_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *tmp;
  gchar *fname=NULL;
  RubricaPreferences *p;
  AddressBook *ab;
  gint page;      /* notebook's page, now active */

  page = rubrica_get_current_page();
  ab = rubrica_get_addressbook(page);

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));
  g_return_if_fail(ab != NULL);

  tmp = gtk_file_selection_get_filename(fs);
  p = rubrica_get_preferences();
  rubrica_set_rubrica(ab->rubrica);

  /* se il file non ha estensione o l'estensione non  .rub */
  if (!rubrica_file_has_extension(tmp))
    fname = g_strdup_printf("%s.rub",tmp);  
  else 
    fname = g_strdup(tmp);
  
  /* se devo effettuare il backup */
  if (p->backup)
    {
      gchar *newname;

      newname = g_strdup_printf("%s1", fname);
      rubrica_do_backup(fname, newname);
      g_free(newname);
    }

  /* se non  stato selezionato nessun file
     o non  possibile aprire il file segnalare l'errore */  
  if ( !fname || !write_xml_file(fname)) 
    {
      gchar *err;
      GtkWidget *dlg;
      
      /* il dialogo di errore ha il widget fs come genitore */
      if (fname)
	err = g_strdup_printf(_("I can't save the file %s"), fname); 
      else
	err = g_strdup_printf(_("I can't save the file")); 
      
      dlg = gnome_error_dialog_parented(err, GTK_WINDOW(fs));
      gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);   
      g_free(err);
    }
  else /* il file  stato salvato */
    {      
      addressbook_set_rubrica(rubrica_get_rubrica(), FALSE, page);
      addressbook_store_info(rubrica_get_rubrica(), FALSE, FALSE, page);
      addressbook_store_path_and_name(g_dirname(fname), g_basename(fname), page);
      
      /* imposto i nuovi filename */
      rubrica_file_set_names(g_dirname(fname), g_basename(fname));
      
      /* se devo mostrare la path del file, aggiorno la label con la path */
      if (preferences_get_show_path())
	rubrica_vista_set_str_label_path(path);
      else
	/* altrimenti la agiorno con il nome del file */
	rubrica_vista_set_str_label_path(nomefile);

      /* in ogni caso, mostro il nome del file */
//      rubrica_vista_set_str_label_file(nomefile);
      
      rubrica_apply_addressbook(page);

      gtk_widget_destroy(GTK_WIDGET(fs));
    }

  g_free(fname);
}

gboolean
write_xml_file(gchar *fname)
{
  xmlDocPtr doc;  /* puntatore al documento xml */
  GList *rub;     /* ptr to rubrica, this contains data that we want save */
  GList *l;       /* alias to rub */

  rub = rubrica_get_rubrica();
  language = preferences_get_language();

  g_return_val_if_fail(fname != NULL, FALSE);
  g_return_val_if_fail(rub != NULL, FALSE);
  
  /* creazione di un nuovo documento xml */
  doc = xmlNewDoc("1.0");
  
  /* creo un nuovo nodo radice rubrica */
  doc->root = xmlNewDocNode(doc, NULL, "Rubrica", NULL);

  /*      Set Language and Get dictionary
   */
  if (strcmp(language, "Italian") == 0)
    {
      xmlNodeSetLang(doc->root,"IT");
      dict = rubrica_get_dictionary("IT");    
    }
  else
    if (strcmp(language, "English") == 0)
      {
	xmlNodeSetLang(doc->root,"EN");
	dict = rubrica_get_dictionary("EN");    
      }
  else
    {
      xmlNodeSetLang(doc->root,"OTHER");
      dict = rubrica_get_dictionary(NULL);    
    }

  /* attraverso la rubrica e scrivo i dati */
  for (l = rub; l != NULL; l = l->next) {
    RubricaItem *item = l->data;

    rubrica_write_elemento_to_xml(doc->root, item);
  }

  if (xmlSaveFile(fname, doc) == -1) 
    {
      xmlFreeDoc(doc);
      return FALSE;
    }  
  /* if we are here the addressbook is write */
  xmlFreeDoc(doc);    
  
  return TRUE;
}

void 
rubrica_write_elemento_to_xml(xmlNodePtr xmlnode, RubricaItem *item)
{
  GList *alias;
  xmlNodePtr newxml;

  xmlNodePtr schedaxml;
  xmlNodePtr datixml;
  xmlNodePtr webxml;
  xmlNodePtr mailxml;
  xmlNodePtr telxml;
  xmlNodePtr notexml;

  xmlNodePtr nodo;

  g_return_if_fail(xmlnode != NULL);
  g_return_if_fail(item != NULL);
  
  /*
    newxml  figlio di xmlnode (il primo xmlnode  doc->root)
    boolxml, schedaxml, datixml, ..., sono figli di newxml
  */
    
  newxml = xmlNewChild(xmlnode, NULL, dict.address, NULL);
  
  /*
    xmlNewTextChild 
    sintassi :
         - nodo genitore del nodo creato con la chiamata
	 - namespace
	 - nome del nodo creato
	 - testo (elemento) da inserire nel nodo creato
  */
  schedaxml = xmlNewTextChild(newxml, NULL, dict.card, item->scheda);

  /* setto le propriet della scheda (editabile, concellabile) */
  if (item->editabile)
    xmlSetProp(schedaxml, dict.mod, "TRUE");
  else
    xmlSetProp(schedaxml, dict.mod, "FALSE");
  if (item->cancellabile)
    xmlSetProp(schedaxml, dict.del, "TRUE");
  else
    xmlSetProp(schedaxml, dict.del, "FALSE");
  
  /* ho creato l'elemento xml es.: 
     <Scheda editabile="TRUE" cancellabile="FALSE>scheda numero uno</Scheda>
     <Card modifying="TRUE" deleting="FALSE">card number one</Card>
  */

  datixml = xmlNewTextChild(newxml, NULL, dict.data, NULL);
  nodo = xmlNewTextChild(datixml, NULL, dict.name, item->nome);
  nodo = xmlNewTextChild(datixml, NULL, dict.surn, item->cognome);
  nodo = xmlNewTextChild(datixml, NULL, dict.prof, item->professione);
  nodo = xmlNewTextChild(datixml, NULL, dict.str, item->via);
  nodo = xmlNewTextChild(datixml, NULL, dict.num, item->numero);
  nodo = xmlNewTextChild(datixml, NULL, dict.zip, item->cap);
  nodo = xmlNewTextChild(datixml, NULL, dict.city, item->citta);
  nodo = xmlNewTextChild(datixml, NULL, dict.prov, item->provincia);
  nodo = xmlNewTextChild(datixml, NULL, dict.st, item->stato);

  webxml = xmlNewChild(newxml, NULL, dict.webadd, NULL);

  alias = item->web_list;
  while(alias)
    {
      nodo = xmlNewTextChild(webxml, NULL, dict.web, alias->data);
      
      alias = alias->next;
    }
  
  mailxml = xmlNewChild(newxml, NULL, dict.mailadd, NULL);
  alias = item->mail_list;
  while(alias)
    {
      nodo = xmlNewTextChild(mailxml, NULL, dict.mail, alias->data);

      alias = alias->next;
    }

  telxml = xmlNewChild(newxml, NULL, dict.tellist, NULL);
  alias = item->tel_list;
  while(alias)
    {
      TelNum *tt = (TelNum*) alias->data;

      nodo = xmlNewTextChild(telxml, NULL, dict.telnum, tt->numero_telefono);

      xmlSetProp(nodo, dict.teltype, tt->tipo);
      
      alias = alias->next;
    }
  
  notexml = xmlNewTextChild(newxml, NULL, dict.note, item->note);
}



/*      Load Routines
*/
gboolean rubrica_real_load_file(gchar *filename)
{
  gboolean use_dict;  
  GList *rub;
  gint page;

  /* if user want use his dictionary */
  use_dict = preferences_get_use_dictionary();

  if (use_dict)
    {
      if(!dictionary_is_loaded())
	rubrica_load_translation();  
      
      /* get dictionary */
      dict = rubrica_get_dictionary(NULL);
    }

  if (!read_xml_file(filename)) 
    return FALSE;
  else
    {
      if (preferences_get_show_path())
	  /*   path and nomefile are globals in file.c
	  */
	  rubrica_vista_set_str_label_path(path);      	
      else
	  rubrica_vista_set_str_label_path(NULL);

      rubrica_vista_set_str_label_file(nomefile);
      rubrica_set_empty(FALSE);      
    }
  /* if we are here, then file has been loaded, so we need to
     store some information about this file, and addressbook 
     strored in memory
  */

  /* get ptr to rubrica (addressbook just read) */
  rub = rubrica_get_rubrica();
  page = rubrica_get_current_page();

  addressbook_store_info(rub,FALSE, FALSE, page);
  addressbook_store_path_and_name(g_dirname(filename),g_basename(filename), page); 
  return TRUE;
}

void rubrica_start_with_file(gchar *path, gchar *file)
{
  gchar *filename;
  gboolean loaded;
  
  filename = g_strdup_printf("%s/%s", path, file);
  rubrica_file_set_start(TRUE);

  loaded = rubrica_real_load_file(filename);
  if (!loaded)
    {
      g_warning("I can't load default addressbook. ");
    }
  
  g_free(filename);
}

void 
rubrica_carica_file(void)
{
  GtkFileSelection *fs;
  GtkWidget *app;

  app = rubrica_get_app();

  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Open file...")));
  
  /* connetto i bottoni ok e cancel */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_apri_file), fs);
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  /* visualizzo la finestra in posizione centrale */
  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);

  /* il genitore della finestra fileselection  app */
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));

  gtk_widget_show(GTK_WIDGET(fs));
}

void 
rubrica_apri_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *fname;
  gchar *tmp;
  gboolean loaded;

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));

  fname = gtk_file_selection_get_filename(fs);
  loaded = rubrica_real_load_file(fname);
  if (!loaded)
    {
      GtkWidget *dlg;
      
      tmp = g_strdup_printf(_("I can't read the file %s"), fname);
      
      dlg = gnome_error_dialog_parented(tmp,GTK_WINDOW(fs));
      gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);

      g_free(tmp);
    }
  else
    gtk_widget_destroy(GTK_WIDGET(fs));
}


gboolean 
read_xml_file(gchar *fname)
{
  xmlDocPtr doc;
  gchar *lang; /* language set in file */

  g_return_val_if_fail(fname != NULL, FALSE);
  
  /* file not exist */
  if (!g_file_exists(fname))
    return FALSE;

  doc = xmlParseFile(fname);

  if (!doc)
    return FALSE;

  if (!doc->root || !doc->root->name || 
      g_strcasecmp(doc->root->name, "Rubrica") != 0)
    {
      xmlFreeDoc(doc);
      return FALSE;
    }
  lang = xmlNodeGetLang(doc->root);

  if(!lang)
    {
      /* may be this file has been created with an rubrica version less then 0.5
	 Those version use english as language
      */
      recover = TRUE;
      g_warning("read_xml_file - recover mode:: salvare il file");     
    }
  else
    {
      if (strcmp(lang, "IT") == 0)
	dict = rubrica_get_dictionary("IT");
      else     
	if (strcmp(lang, "EN") == 0)
	  dict = rubrica_get_dictionary("EN");
	else
	  dict = rubrica_get_dictionary(NULL);
    }

  parse_doc(doc, fname);

  xmlFreeDoc(doc);
 
  rubrica_file_set_names(g_dirname(fname),g_basename(fname)); 
 
  return TRUE;  
}


void 
parse_doc(xmlDocPtr doc, gchar *fname)
{  
  GtkWidget *app;
  xmlNodePtr nodo;
  gint page = 0;

  /* inizializzo le strutture necessarie per 
     caricare una nuova lista
  */
  app = rubrica_get_app();

  if (rubrica_file_is_start())
    rubrica_apply_addressbook(0);
  else
    page = rubrica_do_new_addressbook(app, fname);
    
  rubrica_set_current_page(page);
    

  /* try to load an old or damaged file
   */
  if(recover)
    {  
      xmlNodePtr xmltmp;

      xmltmp = doc->root->childs;
      if (g_strcasecmp(xmltmp->name, "Address") == 0)
	dict = rubrica_get_dictionary("EN");
      if (g_strcasecmp(xmltmp->name, "Indirizzo") == 0)
	dict = rubrica_get_dictionary("IT");
      else
	{	/* procedura disperata, sottoponi all'utente il file */ 

	}
    }

  /* prendo il nodo Indirizzo dal file e lo inserisco nella (nuova) rubrica 
     il primo nodo  doc->root->childs == xmlnew (salvato)
  */
  for(nodo = doc->root->childs; nodo != NULL; nodo=nodo->next)    
    rubrica_load_elemento_from_xml(nodo, NULL);  
}

void
rubrica_load_elemento_from_xml(xmlNodePtr xmlnode, GList *l)
{
  gchar *buffer;
  RubricaItem *r;  
  GList *web, *mail, *tel;
  xmlNodePtr figlio;

  /* i nodi xmlnode sono nodi xmlnew
     - devo estrarre i figli di xmlnew == xmldoc che sono:
       schedaxml, ...
  */

  if (!xmlnode->name ||                               /* se il nodo non ha nome o */
      g_strcasecmp(xmlnode->name, dict.address) != 0) /* il nome non  Indirizzo  */
    return;
    
  r = g_malloc(sizeof(RubricaItem));

  /* schedaxml */
  figlio = xmlnode->childs;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.card) != 0)    
      r->scheda = "";    
  else     
    r->scheda = g_strdup(xmlNodeGetContent(figlio));
  
  /* scheda ha degli attributi, bisogna estrarli */
  buffer = xmlGetProp(figlio, dict.mod);
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->editabile = TRUE;
  else 
    r->editabile = FALSE;
  
  buffer = xmlGetProp(figlio, dict.del);
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->cancellabile = TRUE;
  else 
    r->cancellabile = FALSE;  
  
  /* dati personali -- datixml -- */
  figlio = figlio->next; 
  if (!figlio->name || g_strcasecmp(figlio->name, dict.data) != 0)
    r->nome = r->cognome 
      = r->professione = r->via 
      = r->numero = r->cap 
      = r->citta = r->provincia 
      = r->stato = "";
  else 
    {
      /* carico i figli di datixml
	   - datixml=figlio 
	   - nodo = figlio di datixml
      */
      xmlNodePtr nodo;
      gchar *tmp;

      nodo = figlio->childs;
      tmp = xmlNodeGetContent(nodo);
      if (tmp)
	r->nome = g_strdup(tmp);
      else 
	r->nome = "";
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if (tmp)
	r->cognome = g_strdup(tmp);
      else	
	r->cognome = "";
 
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if (tmp)
	r->professione = g_strdup(tmp);
      else
	r->professione = "";

      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if 
	(tmp) r->via = g_strdup(tmp);
      else      
	r->via = ""; 
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if 
	(tmp) r->numero  = g_strdup(tmp);
      else      
	r->numero = "";
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if 
	(tmp)r->cap = g_strdup(tmp);
	else      
	  r->cap = "";

      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if (tmp)
	r->citta = g_strdup(tmp);
	else      
	  r->citta = "";

      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if (tmp) 
	r->provincia = g_strdup(tmp);
      else     
	r->provincia = "";

      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if (tmp)
	r->stato = g_strdup(tmp);
      else      
	r->stato = "";
    }

  /* web */
  web = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.webadd) != 0)
    {
      r->web_list = NULL;
      r->nweb = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp;

	  tmp = xmlNodeGetContent(nodo);
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      web = g_list_append(web, buffer);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->web_list = web;
      r->nweb = n;
    }
  
  /* email */
  figlio = figlio->next;
  mail = NULL;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.mailadd) != 0)
    {
      r->mail_list = NULL;
      r->nmail = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;

      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp;
	  
	  tmp = xmlNodeGetContent(nodo);
	  
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      mail = g_list_append(mail, buffer);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->mail_list = mail;
      r->nmail = n;
    }  
    
  /* telefono */
  tel = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.tellist) != 0)
    {
      r->tel_list = NULL;
      r->ntel = 0;
    }
  else
    {
      gint n=0;
      xmlNodePtr nodo;
      TelNum *t;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp1=NULL, *tmp2=NULL;
	  
	  tmp1 = xmlNodeGetContent(nodo);
	  tmp2 = xmlGetProp(nodo, dict.teltype);
	  
	  if (tmp1)
	    {	
	      t = g_malloc(sizeof(TelNum));

	      t->numero_telefono = g_strdup(tmp1);
	      t->tipo = g_strdup(tmp2);	  	     
	      
	      tel = g_list_append(tel, t);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->tel_list = tel;
      r->ntel = n;
    }
  
  /* note */
  figlio = figlio->next;
  r->note = NULL;
  if (figlio->name && g_strcasecmp(figlio->name, dict.note) == 0)
    {
      gchar *tmp;
      
      tmp = xmlNodeGetContent(figlio);
      
      if (tmp)
	r->note = g_strdup(tmp);
    }
  
  rubrica_inserisci_elemento(r);
  rubrica_vista_inc_indirizzi();
  rubrica_inserisci_scheda(r);
}

/*  return:
    TRUE  if filename has extension .rub
    FALSE otherwise
*/
gboolean rubrica_file_has_extension(gchar *fn)
{
  gchar *tmp=NULL, *buf;
  gint i, len;

  i = 0;
  buf = g_basename(fn);
  len = strlen(buf);

  while(*buf != '.')
    {
      buf++;
      i++;
            
      if (i == len)     /* end buffer was reached and dot wasn't found */
	return FALSE;	
    }

  tmp = (gchar*) g_malloc(((len - i)*sizeof(gchar))+1);
  strcpy(tmp, buf);
  
  if ((strcmp(tmp,".rub")) == 0)
    {
      g_free(tmp);
      return TRUE;
    }
  else 
    g_free(tmp);
  return FALSE;  
}



/* ******************************************************************* */
/*                                                                     */
/*     routine di compatibilit. Leggono le rubriche create            */
/*     con versioni precedenti alla 0.5                                */
/*                                                                     */
/*     Routine for backward compatibility. Reads old rubrica files     */
/*                                                                     */
/* ******************************************************************* */
void rubrica_apri_old_file(gchar *fname)
{
  gchar *tmp=NULL;
  gint page;

  g_return_if_fail(fname != NULL);

  rubrica_set_empty(TRUE);
  if (!read_old_xml_file(fname)) 
    {
      /* File not opened, error reading file
      */
      GtkWidget *app;
            
      app = rubrica_get_app();
      if (fname)
	tmp = g_strdup_printf(_("I can't read %s"), fname);

      /* if (!fname) function return because fail g_return_... */
      gnome_app_error(GNOME_APP(app), tmp);
      g_free(tmp);
    }
  else /* Ok file read */
      rubrica_set_empty(FALSE);

  page = rubrica_get_current_page();
  addressbook_store_path_and_name(g_dirname(fname),g_basename(fname), page); 
  addressbook_store_info(rubrica_get_rubrica(), FALSE, FALSE, page);
}

gboolean 
read_old_xml_file(gchar *fname)
{
  xmlDocPtr doc;

  g_return_val_if_fail(fname != NULL, FALSE);
  
  /* se il file non esiste */
  if (!g_file_exists(fname))
    return FALSE;

  doc = xmlParseFile(fname);

  if (!doc)
    return FALSE;

  if (!doc->root || !doc->root->name || 
      g_strcasecmp(doc->root->name, "Rubrica") != 0)
    {
      xmlFreeDoc(doc);
      return FALSE;
    }
  parse_old_doc(doc);

  xmlFreeDoc(doc);
 
  return TRUE;  
}

void 
parse_old_doc(xmlDocPtr doc)
{  
  GtkWidget *app;
  xmlNodePtr nodo;
  gint page;

  /* inizializzo le strutture necessarie per caricare una nuova lista
  */
  app = rubrica_get_app();
//  rubrica_crea_nuovo_archivio();
  page = rubrica_do_new_addressbook(app, NULL);

  rubrica_set_current_page(page);
  /* prendo il nodo Indirizzo dal file e lo inserisco nella (nuova) rubrica 
     il primo nodo  doc->root->childs == xmlnew (salvato)
  */
  for(nodo = doc->root->childs; nodo != NULL; nodo=nodo->next)
    rubrica_load_element_from_old_xml(nodo, NULL);  
}

void
rubrica_load_element_from_old_xml(xmlNodePtr xmlnode, GList *l)
{
  gchar *buffer;
  RubricaItem *r;  
  GList *web, *mail, *tel;
  xmlNodePtr figlio;

  /* i nodi xmlnode sono nodi xmlnew
     - devo estrarre i figli di xmlnew == xmldoc che sono:
       schedaxml, ...
  */
  
  if (!xmlnode->name ||  /* se il nodo non ha nome o */
                         /* il nome non  Indirizzo  */
      g_strcasecmp(xmlnode->name, "Indirizzo") != 0) 
    return;
    
  r = g_malloc(sizeof(RubricaItem));

  /* schedaxml */
  figlio = xmlnode->childs;
  if (!figlio->name || g_strcasecmp(figlio->name, "Scheda") != 0)   
    {
      r->scheda = g_strdup("");
    }
  else     
    r->scheda = g_strdup(xmlNodeGetContent(figlio));
    
  /* scheda ha degli attributi, bisogna estrarli */
  buffer = xmlGetProp(figlio, "editabile");
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->editabile = TRUE;
  else 
    r->editabile = FALSE;
  
  buffer = xmlGetProp(figlio, "cancellabile");
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->cancellabile = TRUE;
  else 
    r->cancellabile = FALSE;  
  
  /* dati personali -- datixml -- */
  figlio = figlio->next; 
  if (!figlio->name || g_strcasecmp(figlio->name, "Dati") != 0)
    r->nome = r->cognome 
      = r->professione = r->via 
      = r->numero = r->cap 
      = r->citta = r->provincia 
      = r->stato = g_strdup("");
  else 
    {
      /* carico i figli di datixml
	   - datixml=figlio 
	   - nodo = figlio di datixml
      */
      xmlNodePtr nodo;

      nodo = figlio->childs;
      r->nome = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->cognome = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->professione = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->via = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->numero = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->cap = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->citta = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->provincia = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->stato = g_strdup(xmlNodeGetContent(nodo));
    }

  /* web */
  web = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Indirizzi_Web") != 0)
    {
      r->web_list = NULL;
      r->nweb = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  buffer = g_strdup(xmlNodeGetContent(nodo));
	  
	  web = g_list_append(web, buffer);
	  nodo = nodo->next;
	  n++;
	}
      r->web_list = web;
      r->nweb = n;
    }
  
  /* mail */
  figlio = figlio->next;
  mail = NULL;
  if (!figlio->name || g_strcasecmp(figlio->name, "Indirizzi_Email") != 0)
    {
      r->mail_list = NULL;
      r->nmail = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;

      nodo = figlio->childs;
      while(nodo)
	{
	  buffer = g_strdup(xmlNodeGetContent(nodo));

	  mail = g_list_append(mail, buffer);
	  nodo = nodo->next;
	  n++;
	}
      r->mail_list = mail;
      r->nmail = n;
    }  
    
  /* telefono */
  tel = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Numeri_Telefonici") != 0)
    {
      r->tel_list = NULL;
      r->ntel = 0;
    }
  else
    {
      gint n=0;
      xmlNodePtr nodo;
      TelNum *t;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  t = g_malloc(sizeof(TelNum));
	  
	  t->numero_telefono = g_strdup(xmlNodeGetContent(nodo));
	  t->tipo = g_strdup(xmlGetProp(nodo,"Tipo"));
	  
	  tel = g_list_append(tel, t);
	  nodo = nodo->next;
	  n++;
	}
      r->tel_list = tel;
      r->ntel = n;
    }
  
  /* note */
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Note") != 0)
    r->note = g_strdup("");
  else 
    r->note = g_strdup(xmlNodeGetContent(figlio));
  
  rubrica_inserisci_elemento(r);
  rubrica_vista_inc_indirizzi();
  rubrica_inserisci_scheda(r);
}

